
# q-expansion of newform 9450.2.a.n, downloaded from the LMFDB on 13 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 9450
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[-1], [0], [0], [-1], [0], [1], [-3], [-7], [6], [3], [8], [-2], [6], [-2], [3], [-9], [-6], [-1], [-8], [0], [-14], [-1], [-6], [15], [-8], [18], [4], [12], [2], [6], [7], [6], [-18], [-4], [15], [17], [7], [10], [-24], [12], [15], [11], [0], [-5], [3], [-10], [2], [4], [-12], [-13], [24], [6], [-4], [-6], [-3], [30], [18], [8], [10], [6], [31], [6], [4], [-12], [-8], [15], [20], [-23], [-3], [5], [-18], [18], [10], [22], [-22], [-24], [21], [-5], [12], [-28], [12], [2], [12], [4], [14], [27], [12], [-35], [30], [-17], [-12], [33], [16], [39], [14], [12], [6], [-33], [-11], [2], [40], [45], [0], [-36], [-22], [-20], [0], [3], [6], [44], [28], [28], [-30], [44], [-37], [-48], [31], [3], [15], [-27], [-10], [34], [-42], [-12], [44], [-30], [8], [27], [-26], [31], [-34], [6], [8], [16], [21], [-4], [-12], [4], [0], [54], [-28], [45], [-20], [-15], [-31], [21], [10], [3], [56], [48], [40], [-3], [-20], [0], [28], [12], [32], [9], [16], [24], [-12], [0], [13], [-36], [-6], [-9], [-25], [46], [2], [-12], [24], [2], [42], [-35], [8], [-39], [2], [-18], [46], [11], [55], [6], [10], [51], [-24], [-24], [46], [37], [5], [3], [34], [-27], [20], [-48], [57], [-21], [53], [40], [-39], [-3], [-45], [-64], [-29], [-40], [-48], [12], [20], [24], [36], [-61], [-23], [6], [-23], [36], [24], [8], [52], [-57], [48], [-54], [-40], [-4], [-60], [-47], [-36], [32], [-27], [-36], [-62], [3], [-50], [-43], [53], [-30], [1], [0], [8], [69], [-60], [39], [-57], [-7], [-50], [-46], [21], [-45], [-2], [-18], [-16], [60], [-32], [-39], [-48], [56], [-60], [3], [20], [-44], [54], [-20], [73], [-9], [-31], [-47], [-54], [53], [15], [33], [4], [-9], [5], [46], [46], [-1], [-2], [-2], [57], [20], [38], [-36], [-72], [41], [-63], [47], [52], [-36], [-59], [42], [-46], [-81], [87], [66], [24], [78], [-26], [54], [-28], [42], [78], [28], [34], [15], [80], [84], [-4], [-2], [9], [50], [-72], [-50], [-75], [-42], [42], [22], [60], [-58], [-42], [-24], [-8], [-18], [-7], [-59], [-18], [79], [6], [38], [-40], [28], [54], [39], [20], [-30], [80], [-24], [-58], [18], [-70], [-51], [62], [-2], [-80], [48], [-30], [-73], [54], [81], [-25], [58], [-39], [-60], [77], [73], [-15], [-14], [62], [-21], [3], [-54], [-72], [6], [-62], [15], [-66], [-69], [-65], [-17], [30], [-56], [-55], [15], [56], [48], [-99], [-46], [-56], [-96], [42], [-62], [-45], [-98], [-78], [-48], [-44], [60], [-79], [-3], [89], [-74], [40], [63], [-73], [-78], [-39], [43], [24], [22], [-100], [75], [-22], [-42], [-85], [60], [88], [42], [-30], [-88], [-44], [6], [64], [24], [-50], [87], [-105], [-22], [13], [18], [-30], [-26], [66], [-3], [78], [22], [48], [54], [-38], [12], [48], [-24], [53], [99], [68], [12], [-22], [0], [-62], [57], [47], [92], [-8], [26], [-42], [-102], [-10], [-18], [-64], [-24], [-92], [0], [-34], [-10], [-68], [-33], [51], [54], [19], [27], [80], [78], [-29], [24], [44], [17], [36], [26], [67], [46], [56], [0], [0], [-52], [-107], [-57], [75], [17], [12], [94], [-90], [-82], [3], [42], [40], [-99], [106], [-18], [-68], [60], [-58], [18], [95], [-16], [-107], [93], [11], [-72], [60], [-103], [-20], [-33], [50], [-10], [-30], [-14], [-108], [-20], [-56], [-54], [-60], [-76], [88], [1], [-24], [-93], [-110], [-45], [-100], [-17], [-72], [2], [-108], [-32], [-8], [-34], [30], [-12], [44], [-108], [22], [-12], [111], [84], [-44], [-102], [4], [75], [-53], [96], [-17], [21], [14], [88], [39], [-81], [44], [-102], [102], [50], [-95], [42], [-2], [114], [24], [28], [33], [93], [33], [11], [16], [105], [-1], [-29], [33], [57], [12], [19], [86], [-37], [0], [-58], [-60], [27], [31], [-36], [8], [16], [86], [75], [-6], [5], [87], [-70], [75], [16], [66], [108], [62], [78], [46], [-24], [105], [-128]]
